"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
const fg = require("fast-glob");
const sfpowerkit_1 = require("../../../../sfpowerkit");
var path = require("path");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "apextestsuite_convert");
class Convert extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        const entries = fg.sync(`**${this.flags.name}.testSuite-meta.xml`, {
            onlyFiles: true,
            absolute: true,
            baseNameMatch: true
        });
        if (!entries[0])
            throw new core_1.SfdxError(`Apex Test Suite ${this.flags.name} not found`);
        sfpowerkit_1.SFPowerkit.log(`Apex Test Suite File Path ${entries[0]}`, core_1.LoggerLevel.DEBUG);
        if (fs_extra_1.default.existsSync(path.resolve(entries[0]))) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            let apex_test_suite = await parseString(fs_extra_1.default.readFileSync(path.resolve(entries[0])));
            let apex_test_suite_as_string = JSON.stringify(apex_test_suite.ApexTestSuite.testClassName);
            apex_test_suite_as_string = apex_test_suite_as_string.replace(/("|')/g, "");
            apex_test_suite_as_string = apex_test_suite_as_string.slice(1, -1);
            apex_test_suite_as_string = '"' + apex_test_suite_as_string.concat('"');
            this.ux.log(apex_test_suite_as_string);
            return apex_test_suite_as_string;
        }
        else {
            throw new core_1.SfdxError("Apex Test Suite not found");
        }
    }
}
exports.default = Convert;
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Convert.requiresProject = true;
Convert.description = messages.getMessage("commandDescription");
Convert.examples = [
    `$ sfdx sfpowerkit:source:apextestsuite:convert -n MyApexTestSuite 
    "ABC2,ABC1Test"    
  `
];
Convert.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription")
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
//# sourceMappingURL=convert.js.map