"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sfpowerkit_1 = require("../../../sfpowerkit");
const core_1 = require("@salesforce/core");
class PackageInfo {
    constructor(conn, apiversion, jsonOutput) {
        this.conn = conn;
        this.apiversion = apiversion;
        this.jsonOutput = jsonOutput;
    }
    async getPackages() {
        //await this.getInstalledPackageInfo();
        let packageDetails = await this.getInstalledPackages();
        sfpowerkit_1.SFPowerkit.log(packageDetails, core_1.LoggerLevel.DEBUG);
        return packageDetails;
    }
    async getInstalledPackages() {
        let packageDetails = [];
        let installedPackagesQuery = "SELECT Id, SubscriberPackageId, SubscriberPackage.NamespacePrefix, SubscriberPackage.Name, " +
            "SubscriberPackageVersion.Id, SubscriberPackageVersion.Name, SubscriberPackageVersion.MajorVersion, SubscriberPackageVersion.MinorVersion, " +
            "SubscriberPackageVersion.PatchVersion, SubscriberPackageVersion.BuildNumber FROM InstalledSubscriberPackage " +
            "ORDER BY SubscriberPackageId";
        let packageNamespacePrefixList = [];
        await this.conn.tooling.query(installedPackagesQuery).then(queryResult => {
            const records = queryResult.records;
            if (records && records.length > 0) {
                records.forEach(record => {
                    const packageDetail = {};
                    packageDetail.packageName = record["SubscriberPackage"]["Name"];
                    packageDetail.packageNamespacePrefix =
                        record["SubscriberPackage"]["NamespacePrefix"];
                    packageDetail.packageVersionId =
                        record["SubscriberPackageVersion"]["Id"];
                    packageDetail.packageVersionNumber = `${record["SubscriberPackageVersion"]["MajorVersion"]}.${record["SubscriberPackageVersion"]["MinorVersion"]}.${record["SubscriberPackageVersion"]["PatchVersion"]}.${record["SubscriberPackageVersion"]["BuildNumber"]}`;
                    packageDetails.push(packageDetail);
                    if (packageDetail.packageNamespacePrefix) {
                        packageNamespacePrefixList.push("'" + packageDetail.packageNamespacePrefix + "'");
                    }
                });
            }
        });
        let licenseMap = new Map();
        if (packageNamespacePrefixList) {
            let packageLicensingQuery = `SELECT AllowedLicenses, UsedLicenses,ExpirationDate, NamespacePrefix, IsProvisioned, Status FROM PackageLicense  WHERE NamespacePrefix IN (${packageNamespacePrefixList})`;
            await this.conn.query(packageLicensingQuery).then(queryResult => {
                if (queryResult.records && queryResult.records.length > 0) {
                    queryResult.records.forEach(record => {
                        let licenseDetailObj = {};
                        licenseDetailObj.allowedLicenses = record["AllowedLicenses"];
                        licenseDetailObj.usedLicenses = record["UsedLicenses"];
                        licenseDetailObj.expirationDate = record["ExpirationDate"];
                        licenseDetailObj.status = record["Status"];
                        licenseMap.set(record["NamespacePrefix"], licenseDetailObj);
                    });
                }
            });
        }
        if (packageDetails && licenseMap) {
            packageDetails.forEach(detail => {
                if (detail.packageNamespacePrefix &&
                    licenseMap.has(detail.packageNamespacePrefix)) {
                    let licDetail = licenseMap.get(detail.packageNamespacePrefix);
                    detail.allowedLicenses = licDetail.allowedLicenses;
                    detail.usedLicenses = licDetail.usedLicenses;
                    detail.expirationDate = licDetail.expirationDate;
                    detail.status = licDetail.status;
                }
            });
        }
        return packageDetails;
    }
}
exports.default = PackageInfo;
//# sourceMappingURL=packageInfo.js.map