"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const xml2js = __importStar(require("xml2js"));
const util = require("util");
const diffUtil_1 = __importDefault(require("./diffUtil"));
const _ = require("lodash");
const parser = new xml2js.Parser({
    explicitArray: false,
    valueProcessors: [
        function (name) {
            if (name === "true")
                name = true;
            if (name === "false")
                name = false;
            return name;
        }
    ]
});
class CustomLabelsDiff {
    static async getMembers(filePath) {
        let fileContent = fs.readFileSync(filePath, "utf8").toString();
        const parseString = util.promisify(parser.parseString);
        let members = [];
        if (fileContent !== "") {
            let parseResult = await parseString(fileContent);
            let customLabelsObj = parseResult.CustomLabels || {};
            if (!_.isNil(customLabelsObj.labels)) {
                if (!Array.isArray(customLabelsObj.labels)) {
                    members.push(customLabelsObj.labels.fullName);
                }
                else {
                    members = customLabelsObj.labels.map(label => {
                        return label.fullName;
                    });
                }
            }
        }
        return members;
    }
    static async generateCustomLabelsXml(customLabelsXml1, customLabelsXml2, outputFilePath, destructivePackageObj, resultOutput, isDestructive) {
        let customLabelsObj1 = {};
        let customLabelsObj2 = {};
        const parseString = util.promisify(parser.parseString);
        if (customLabelsXml1 !== "") {
            let parseResult = await parseString(customLabelsXml1);
            customLabelsObj1 = parseResult.CustomLabels || {};
        }
        if (customLabelsXml2 !== "") {
            let parseResult = await parseString(customLabelsXml2);
            customLabelsObj2 = parseResult.CustomLabels || {};
        }
        // Building the new workflow object for the added and modified fields
        let addedEditedOrDeleted = CustomLabelsDiff.buildCustomLabelsObj(customLabelsObj1, customLabelsObj2);
        if (addedEditedOrDeleted.addedEdited.labels &&
            addedEditedOrDeleted.addedEdited.labels.length > 0) {
            CustomLabelsDiff.writeCustomLabel(addedEditedOrDeleted.addedEdited, outputFilePath);
        }
        // Check for deletion
        destructivePackageObj = CustomLabelsDiff.buildDestructiveChanges(addedEditedOrDeleted.deleted, destructivePackageObj);
        CustomLabelsDiff.updateOutput(addedEditedOrDeleted.addedEdited, resultOutput, "Deploy", outputFilePath);
        if (isDestructive) {
            CustomLabelsDiff.updateOutput(addedEditedOrDeleted.deleted, resultOutput, "Delete", "destructiveChanges.xml");
        }
        return destructivePackageObj;
    }
    static updateOutput(customLabelObj, resultOutput, action, filePath) {
        customLabelObj.labels.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "CustomLabel",
                componentName: elem.fullName,
                path: filePath
            });
        });
    }
    static buildCustomLabelsObj(customLabelsObj1, customLabelsObj2) {
        let newcustomLabelsObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            labels: []
        };
        if (!_.isNil(customLabelsObj1.labels) &&
            !Array.isArray(customLabelsObj1.labels)) {
            customLabelsObj1.labels = [customLabelsObj1.labels];
        }
        if (!_.isNil(customLabelsObj2.labels) &&
            !Array.isArray(customLabelsObj2.labels)) {
            customLabelsObj2.labels = [customLabelsObj2.labels];
        }
        let deletedCustomLabelsObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            labels: []
        };
        let addedDeleted = diffUtil_1.default.getChangedOrAdded(customLabelsObj1.labels, customLabelsObj2.labels, "fullName");
        newcustomLabelsObj.labels = addedDeleted.addedEdited;
        deletedCustomLabelsObj.labels = addedDeleted.deleted;
        return {
            addedEdited: newcustomLabelsObj,
            deleted: deletedCustomLabelsObj
        };
    }
    static buildDestructiveChanges(deletedCustomLabels, destructivePackageObj) {
        let labelType = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "CustomLabel";
        });
        if (labelType === undefined &&
            deletedCustomLabels.labels !== undefined &&
            deletedCustomLabels.labels.length > 0) {
            labelType = {
                name: "CustomLabel",
                members: []
            };
            destructivePackageObj.push(labelType);
        }
        if (deletedCustomLabels.labels !== undefined) {
            deletedCustomLabels.labels.forEach(elem => {
                labelType.members.push(elem.fullName);
            });
        }
        return destructivePackageObj;
    }
    static writeCustomLabel(newCustomLabelsObj, outputFilePath) {
        const builder = new xml2js.Builder({
            xmldec: { version: "1.0", encoding: "UTF-8", standalone: null }
        });
        let customLabelObj = {
            CustomLabels: newCustomLabelsObj
        };
        let xml = builder.buildObject(customLabelObj);
        fs.writeFileSync(outputFilePath, xml);
    }
}
exports.default = CustomLabelsDiff;
//# sourceMappingURL=customLabelsDiff.js.map